package com.ruoyi.ss.commandLog.service.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.commandLog.mapper.CommandLogMapper;
import com.ruoyi.ss.commandLog.service.ICommandLogService;
import com.ruoyi.ss.commandLog.domain.CommandLog;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.service.DeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 命令日志Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-07-06
 */
@Service
@Slf4j
public class CommandLogServiceImpl implements ICommandLogService
{
    @Autowired
    private CommandLogMapper commandLogMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceService deviceService;

    /**
     * 查询命令日志
     *
     * @param id 命令日志主键
     * @return 命令日志
     */
    @Override
    public CommandLog selectEtCommandLogById(Long id)
    {
        return commandLogMapper.selectEtCommandLogById(id);
    }

    /**
     * 查询命令日志列表
     *
     * @param eCommandLog 命令日志
     * @return 命令日志
     */
    @Override
    public List<CommandLog> selectEtCommandLogList(CommandLog eCommandLog)
    {
        return commandLogMapper.selectEtCommandLogList(eCommandLog);
    }

    /**
     * 新增命令日志
     *
     * @param eCommandLog 命令日志
     * @return 结果
     */
    @Override
    public int insertEtCommandLog(CommandLog eCommandLog)
    {
        return commandLogMapper.insertEtCommandLog(eCommandLog);
    }

    /**
     * 修改命令日志
     *
     * @param eCommandLog 命令日志
     * @return 结果
     */
    @Override
    public int updateEtCommandLog(CommandLog eCommandLog)
    {
        return commandLogMapper.updateEtCommandLog(eCommandLog);
    }

    /**
     * 批量删除命令日志
     *
     * @param ids 需要删除的命令日志主键
     * @return 结果
     */
    @Override
    public int deleteEtCommandLogByIds(Long[] ids)
    {
        return commandLogMapper.deleteEtCommandLogByIds(ids);
    }

    /**
     * 删除命令日志信息
     *
     * @param id 命令日志主键
     * @return 结果
     */
    @Override
    public int deleteEtCommandLogById(Long id)
    {
        return commandLogMapper.deleteEtCommandLogById(id);
    }

    /**
     * 异步保存日志
     */
    @Override
    public void asynchronousSaveLog(String command, String mac, String result, String type, LoginUser user) {
        //异步保存发送命令日志
        scheduledExecutorService.schedule(() -> {
            CommandLog etCommandLog = new CommandLog();
            etCommandLog.setCommand(command);
            etCommandLog.setType(type);
            etCommandLog.setMac(mac);
            etCommandLog.setSn(deviceService.selectSnByMac(mac));
            etCommandLog.setResult(result);
            etCommandLog.setCreateTime(DateUtils.getNowDate());
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            etCommandLog.setCallStatus(code);
            etCommandLog.setUserId(user.getUserId());
            etCommandLog.setUserName(user.getUsername());
            int i = commandLogMapper.insertEtCommandLog(etCommandLog);
            if(i>0){
                log.info("【发送命令】异步保存发送命令日志");
            }
        }, 0, TimeUnit.SECONDS);
    }
}
