package com.ruoyi.ss.toilet.service.impl;

import java.util.List;

import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.store.domain.StoreVO;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.toilet.domain.Toilet;
import com.ruoyi.ss.toilet.domain.ToiletQuery;
import com.ruoyi.ss.toilet.domain.ToiletVO;
import com.ruoyi.ss.toilet.mapper.ToiletMapper;
import com.ruoyi.ss.toilet.service.IToiletService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static com.ruoyi.common.constant.ServiceConstants.LOCK_MODE_POWER_OFF;
import static com.ruoyi.common.constant.ServiceConstants.LOCK_MODE_POWER_ON;
import static com.ruoyi.common.utils.SecurityUtils.getUserId;

/**
 * 卫生间Service业务层处理
 *
 * @author qzz
 * @date 2025-01-07
 */
@Service
public class ToiletServiceImpl implements IToiletService
{
    @Autowired
    private ToiletMapper toiletMapper;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private DeviceService deviceService;

    /**
     * 查询卫生间
     *
     * @param toiletId 卫生间主键
     * @return 卫生间
     */
    @Override
    public ToiletVO selectToiletByToiletId(Long toiletId)
    {
        ToiletVO toiletVO = toiletMapper.selectToiletByToiletId(toiletId);
        Long deviceId = toiletVO.getDeviceId();
        if(deviceId != null){
            toiletVO.setDevice(deviceService.selectById(deviceId));
        }
        return toiletVO;
    }

    /**
     * 查询卫生间列表
     *
     * @param toilet 卫生间
     * @return 卫生间
     */
    @Override
    public List<ToiletVO> selectToiletList(ToiletQuery toilet)
    {
        return toiletMapper.selectToiletList(toilet);
    }

    /**
     * 新增卫生间
     *
     * @param toilet 卫生间
     * @return 结果
     */
    @Override
    public int insertToilet(Toilet toilet)
    {
        StoreVO storeVO = storeService.selectEStoreByStoreId(toilet.getStoreId());
        ServiceUtil.assertion(storeVO == null,"店铺不存在");
        toilet.setStoreName(storeVO.getName());
        return toiletMapper.insertToilet(toilet);
    }

    /**
     * 修改卫生间
     *
     * @param toilet 卫生间
     * @return 结果
     */
    @Override
    public int updateToilet(Toilet toilet)
    {
        return toiletMapper.updateToilet(toilet);
    }

    /**
     * 批量删除卫生间
     *
     * @param toiletIds 需要删除的卫生间主键
     * @return 结果
     */
    @Override
    public int deleteToiletByToiletIds(Long[] toiletIds)
    {
        return toiletMapper.deleteToiletByToiletIds(toiletIds);
    }

    /**
     * 删除卫生间信息
     *
     * @param toiletId 卫生间主键
     * @return 结果
     */
    @Override
    public int deleteToiletByToiletId(Long toiletId)
    {
        return toiletMapper.deleteToiletByToiletId(toiletId);
    }

    /**
     * 下架设备（根据设备id）
     * @param deviceId 设备id
     * @return String
     */
    @Override
    public int offlineDeviceByDeviceId(Long deviceId) {
        return toiletMapper.offlineDeviceByDeviceId(deviceId);
    }

    /**
     * 查询数量
     * @param toilet 查询条件
     * @return String
     */
    @Override
    public int selectCount(ToiletQuery toilet) {
        return toiletMapper.selectCount(toilet);
    }

    /**
     * 开卫生间门
     * @param toiletVO 卫生间对象
     */
    @Override
    public boolean openToilet(ToiletVO toiletVO) {
        ServiceUtil.assertion(toiletVO == null, "卫生间不存在");
        return deviceService.controlDevice(toiletVO.getDeviceId(), toiletVO.getUnlockTime(), toiletVO.getUnlockMode(), true);
    }

}
